/*
 * Setting the base menu-wise variables.
 */
var ext_holder;
var ext_uses_browser = false;
if (typeof browser !== 'undefined') {
  ext_holder = browser;
  ext_uses_browser = true;
} else {
  ext_holder = chrome;
}

/*
 * Auxiliary methods.
 */
var no_op_function = function () {
};

var err_output = function (data) {
  console.log(data);
};

var check_last_error = function () {
  if (ext_holder.runtime.lastError) {
    console.log(ext_holder.runtime.lastError);
  }
};

var context_menu_id_base_url = "reverse-image-search-url";
var context_menu_id_base_file = "reverse-image-search-file";
var context_menu_id_base_flip = "reverse-image-search-flip";
var context_menu_id_base_gray = "reverse-image-search-gray";

ext_holder.contextMenus.create({
  "id": context_menu_id_base_url,
  "title": ext_holder.i18n.getMessage("menuItemSearchURL"),
  "type" : "normal",
  "targetUrlPatterns": ["*://*/*"],
  "contexts": ["image"]
}, check_last_error);

ext_holder.contextMenus.create({
  "id": context_menu_id_base_file,
  "title": ext_holder.i18n.getMessage("menuItemSearchFile"),
  "type" : "normal",
  "targetUrlPatterns": ["*://*/*"],
  "contexts": ["image"]
}, check_last_error);

ext_holder.contextMenus.create({
  "id": context_menu_id_base_flip,
  "title": ext_holder.i18n.getMessage("menuItemSearchFlip"),
  "type" : "normal",
  "targetUrlPatterns": ["*://*/*"],
  "contexts": ["image"]
}, check_last_error);

ext_holder.contextMenus.create({
  "id": context_menu_id_base_gray,
  "title": ext_holder.i18n.getMessage("menuItemSearchGray"),
  "type" : "normal",
  "targetUrlPatterns": ["*://*/*"],
  "contexts": ["image"]
}, check_last_error);

/*
 * Menus for the search engines that provide reverse image search.
 */
var search_engines = {
  baidu: "Baidu",
  bing: "Bing",
  google: "Google",
  tineye: "TinEye",
  yandex: "Yandex"
};
var one_engine;
for (one_engine in search_engines) {
  if (!(search_engines.hasOwnProperty(one_engine))) {
    continue;
  }
  ext_holder.contextMenus.create({
    "id": context_menu_id_base_url + "-via-" + one_engine,
    "parentId": context_menu_id_base_url,
    "title": ext_holder.i18n.getMessage("menuItemVia" + search_engines[one_engine]),
    "type" : "normal",
    "targetUrlPatterns": ["*://*/*"],
    "contexts": ["image"]
  }, check_last_error);

  ext_holder.contextMenus.create({
    "id": context_menu_id_base_file + "-via-"  + one_engine,
    "parentId": context_menu_id_base_file,
    "title": ext_holder.i18n.getMessage("menuItemVia" + search_engines[one_engine]),
    "type" : "normal",
    "targetUrlPatterns": ["*://*/*"],
    "contexts": ["image"]
  }, check_last_error);

  ext_holder.contextMenus.create({
    "id": context_menu_id_base_flip + "-via-"  + one_engine,
    "parentId": context_menu_id_base_flip,
    "title": ext_holder.i18n.getMessage("menuItemVia" + search_engines[one_engine]),
    "type" : "normal",
    "targetUrlPatterns": ["*://*/*"],
    "contexts": ["image"]
  }, check_last_error);

  ext_holder.contextMenus.create({
    "id": context_menu_id_base_gray + "-via-"  + one_engine,
    "parentId": context_menu_id_base_gray,
    "title": ext_holder.i18n.getMessage("menuItemVia" + search_engines[one_engine]),
    "type" : "normal",
    "targetUrlPatterns": ["*://*/*"],
    "contexts": ["image"]
  }, check_last_error);
}

/*
 * Making requests for the search when done via URLs.
 */
var reverse_image_search_by_url = function (via_engine, image_url, tab_idx) {
  image_url = encodeURIComponent(image_url);
  var search_url = "";
  var pref_lang = "en";
  if (navigator.language) {
    pref_lang = navigator.language;
  }
  if (navigator.languages && navigator.languages[0]) {
    pref_lang = navigator.languages[0];
  }
  pref_lang = pref_lang.split("-")[0].toLowerCase();

  var yandex_tld = "com";
  var yandex_cases = {"ru":"ru", "be":"by", "uk":"ua", "kk":"kz", "tr":"com.tr"};
  if (pref_lang in yandex_cases) {
    yandex_tld = yandex_cases[pref_lang];
  }

  switch (via_engine) {
    case "baidu":
      search_url = "https://image.baidu.com/n/pc_search?queryImageUrl=" + image_url;
      break;
    case "bing":
      search_url = "https://www.bing.com/images/searchbyimage?imgurl=" + image_url + "&cbir=sbi&FORM=IRSBIQ";
      break;
    case "google":
      search_url = "https://images.google.com/searchbyimage?image_url=" + image_url;
      break;
    case "tineye":
      search_url = "https://tineye.com/search?url=" + image_url;
      break;
    case "yandex":
      search_url = "https://yandex." + yandex_tld +"/images/search?url=" + image_url + "&rpt=imageview";
      break;
    default:
      break;
  }

  if (search_url != "") {
    if (ext_uses_browser) {
      ext_holder.tabs.create({url: search_url, active: true, index: (tab_idx+1)}).then(no_op_function, err_output);
    } else {
      ext_holder.tabs.create({url: search_url, active: true, index: (tab_idx+1)}, check_last_error);
    }
  }
};

/*
 * Making requests for the search when done via image data.
 */
var reverse_image_search_by_data = function (url_suffix, tab_idx) {
  var search_url = ext_holder.runtime.getURL("search.html");

  if (ext_uses_browser) {
    ext_holder.tabs.create({url: search_url + "#" + url_suffix, active: true, index: (tab_idx+1)}).then(no_op_function, err_output);
  } else {
    ext_holder.tabs.create({url: search_url + "#" + url_suffix, active: true, index: (tab_idx+1)}, check_last_error);
  }
};

/*
 * Listening on the menu item clicks.
 */
ext_holder.contextMenus.onClicked.addListener((info, tab) => {

  if (info.menuItemId.substring(0, context_menu_id_base_url.length)==context_menu_id_base_url) {
    if (info.srcUrl.substring(0, 4)=="http") {
      reverse_image_search_by_url(info.menuItemId.split("-").pop(), info.srcUrl, tab.index);
    }
    return;
  }
  if (info.menuItemId.substring(0, context_menu_id_base_file.length)==context_menu_id_base_file) {
    if (info.srcUrl.substring(0, 4)=="http") {
      reverse_image_search_by_data("file:orig#" + info.menuItemId.split("-").pop() + "#" + info.srcUrl, tab.index);
    }
    return;
  }
  if (info.menuItemId.substring(0, context_menu_id_base_flip.length)==context_menu_id_base_flip) {
    if (info.srcUrl.substring(0, 4)=="http") {
      reverse_image_search_by_data("file:flip#" + info.menuItemId.split("-").pop() + "#" + info.srcUrl, tab.index);
    }
    return;
  }
  if (info.menuItemId.substring(0, context_menu_id_base_gray.length)==context_menu_id_base_gray) {
    if (info.srcUrl.substring(0, 4)=="http") {
      reverse_image_search_by_data("file:gray#" + info.menuItemId.split("-").pop() + "#" + info.srcUrl, tab.index);
    }
    return;
  }

});

