var ext_holder;

// debugging output, and informing the user;

var under_debug = false;

var no_op_function = function () {
};

var debug_output = function (data) {
  if (under_debug) {
    console.log(data);
  }
};

var do_log_error = function (error_data) {
  debug_output(error_data);
};

var do_log_last_error = function () {
  if (ext_holder.runtime.lastError) {
    debug_output(ext_holder.runtime.lastError);
  }
};

var outer_message = function (mssg, expl) {
  var mssg_el = document.getElementById("message-el");
  if (mssg_el) {
    if (!mssg) {
      mssg = "";
    }
    mssg_el.innerHTML = mssg;
  }
  var expl_el = document.getElementById("message-expl-el");
  if (expl_el) {
    if ((typeof expl === "undefined") || (!expl)) {
      expl = "";
    }
    expl_el.innerHTML = expl;
  }
};

// setting initial variables

var ext_uses_browser = false;
var ext_uses_chrome = false;
if (typeof browser !== "undefined") {
  ext_holder = browser;
  ext_uses_browser = true;
} else {
  ext_holder = chrome;
  ext_uses_chrome = true;
}

var pref_lang = "en";
if (navigator.language) {
  pref_lang = navigator.language;
}
if (navigator.languages && navigator.languages[0]) {
  pref_lang = navigator.languages[0];
}
pref_lang = pref_lang.split("-")[0].toLowerCase();

var yandex_tld = "com";
var yandex_cases = {"ru":"ru", "be":"by", "uk":"ua", "kk":"kz", "tr":"com.tr"};
if (pref_lang in yandex_cases) {
  yandex_tld = yandex_cases[pref_lang];
}

var can_do_post = false;
if (navigator && navigator.appVersion) {
  if (navigator.appVersion.toLowerCase().indexOf("AppleWebKit".toLowerCase()) >= 0) {
    can_do_post = true;
  }
}
if (navigator && navigator.userAgent) {
  if (navigator.userAgent.toLowerCase().indexOf("AppleWebKit".toLowerCase()) >= 0) {
    can_do_post = true;
  }
}

var read_browser_info = function (info) {
  if (info && info.name) {
    if (info.name.toLowerCase() == "Firefox".toLowerCase()) {
      can_do_post = false;
    }
  }
};

if ((typeof browser !== "undefined") && browser && browser.runtime && browser.runtime.getBrowserInfo) {
  browser.runtime.getBrowserInfo().then(read_browser_info, do_log_error);
}

// in the case of dealing with Bing under Firefox, we have an issue,
// thus informing the user on that (unless asked to not warn on that);
// this makes most of the UI complexity here; the cause is a FF bug:
// https://bugzilla.mozilla.org/show_bug.cgi?id=1344465

var ask_not_on_data_uri_key = "do_not_show_data_uri_warning";

var submit_to_bing_via_data_uri_set = null;

var submit_to_bing_via_data_uri = function () {
  debug_output("submit_to_bing_via_data_uri");

  if (!submit_to_bing_via_data_uri_set) {
    outer_message("Issue during page processing");
    return;
  }

  outer_message("Submitting image data");
  try {
    window.location.replace(submit_to_bing_via_data_uri_set);
  } catch (e) {
    // FF does not show any message here, as it apparently considered it an issue within the new page;
    outer_message("Top-level data URIs probably disallowed", "security.data_uri.block_toplevel_data_uri_navigations");
  }
};

var submit_to_bing_via_data_uri_action = function (evt) {
  debug_output("submit_to_bing_via_data_uri_action");

  evt = evt || window.event;
  if (evt.preventDefault) {evt.preventDefault();}
  if (evt.stopPropagation) {evt.stopPropagation();}

  submit_to_bing_via_data_uri();
  return false;
};

var to_ask_on_data_uri_on_call = function (item_got) {
  debug_output("to_ask_on_data_uri_on_call");

  if (ext_holder.runtime.lastError) {
    debug_output(ext_holder.runtime.lastError);
  }

  to_ask_on_data_uri_on_got(item_got);
}

var to_ask_on_data_uri_on_got = function (item_got) {
  debug_output("to_ask_on_data_uri_on_got");

  var should_ask = true;
  if (item_got && (ask_not_on_data_uri_key in item_got) && item_got[ask_not_on_data_uri_key]) {
    should_ask = false;
  }
  if (!should_ask) {
    // continue with sending
    submit_to_bing_via_data_uri();
    return;
  }

  // ask user;
  outer_message("Action requested");
  var els_to_show = [
    "bing-parts-holder"
  ];
  var aux_el;
  for (var ind = els_to_show.length - 1; ind >= 0; ind--) {
    aux_el = document.getElementById(els_to_show[ind]);
    if (aux_el) {
      aux_el.style.display = "block";
    } else {
      outer_message("Error during presenting information");
    }
  }
};

var to_ask_on_data_uri_on_error = function (item_error) {
  debug_output("to_ask_on_data_uri_on_error");

  debug_output(item_error);

  // nothing found, have to ask;
  to_ask_on_data_uri_on_got(null);
};

var should_ask_user_on_data_uri = function () {
  debug_output("should_ask_user_on_data_uri");

  if (ext_uses_browser) {
    ext_holder.storage.local.get(ask_not_on_data_uri_key).then(to_ask_on_data_uri_on_got, to_ask_on_data_uri_on_error);
  } else {
    ext_holder.storage.local.get(ask_not_on_data_uri_key, to_ask_on_data_uri_on_call);
  }

};

var next_on_bing_data_uri_to_not_ask = function (value) {
  debug_output("next_on_bing_data_uri_to_not_ask");

  var set_param = {};
  set_param[ask_not_on_data_uri_key] = value;

  if (ext_uses_browser) {
    ext_holder.storage.local.set(set_param).then(no_op_function, do_log_error);
  } else {
    ext_holder.storage.local.set(set_param, do_log_last_error);
  }

  var not_ask_cb_el = document.getElementById("bing-not-ask-cb");
  if (not_ask_cb_el) {
    not_ask_cb_el.checked = value;
  }
};

var next_on_bing_data_uri_to_not_ask_revert_call = function (item_got) {
  debug_output("next_on_bing_data_uri_to_not_ask_revert_call");

  if (ext_holder.runtime.lastError) {
    debug_output(ext_holder.runtime.lastError);
  }

  next_on_bing_data_uri_to_not_ask_revert_got(item_got);
};

var next_on_bing_data_uri_to_not_ask_revert_got = function (item_got) {
  debug_output("next_on_bing_data_uri_to_not_ask_revert_got");

  var avoid_ask = false;
  if (item_got && (ask_not_on_data_uri_key in item_got) && item_got[ask_not_on_data_uri_key]) {
    avoid_ask = true;
  }
  next_on_bing_data_uri_to_not_ask(!avoid_ask);
};

var next_on_bing_data_uri_to_not_ask_revert_error = function (item_error) {
  debug_output("next_on_bing_data_uri_to_not_ask_revert_error");

  debug_output(item_error);

  next_on_bing_data_uri_to_not_ask(true);
};

var next_on_bing_data_uri_to_not_ask_action = function (evt) {
  debug_output("next_on_bing_data_uri_to_not_ask_action");

  evt = evt || window.event;
  if (evt.preventDefault) {evt.preventDefault();}
  if (evt.stopPropagation) {evt.stopPropagation();}

  if (ext_uses_browser) {
    ext_holder.storage.local.get(ask_not_on_data_uri_key).then(next_on_bing_data_uri_to_not_ask_revert_got, next_on_bing_data_uri_to_not_ask_revert_error);
  } else {
    ext_holder.storage.local.get(ask_not_on_data_uri_key, next_on_bing_data_uri_to_not_ask_revert_call);
  }

  return false;
};

// in the case of AJAX-based submitting of image data to search provider,
// we look at the returned data, and redirecting to the respective URL;

var redir_to_img_search = function (redir_url) {
  debug_output("redir_to_img_search");
  outer_message("Redirecting to the search results");

  window.location.replace(redir_url);
};

var img_upload = null;
var img_upload_runs = false;

var utilize_image_upload = function (ans_state, search_prov, aux_data) {
  debug_output("utilize_image_upload");

  img_upload_runs = false;
  if (!img_upload) {
    return;
  }
  if (!ans_state) {
    img_upload = null;
    return;
  }

  if (search_prov=="baidu") {
    var use_baidu_response = img_upload.response;
    var use_baidu_stored = null;
    var use_baidu_sign = null;
    if (!use_baidu_response) {
      outer_message("No response data from Baidu");
      return;
    }
    try {
      use_baidu_response = JSON.parse(use_baidu_response);
      if (use_baidu_response) {
        use_baidu_stored = use_baidu_response["url"];
        if (use_baidu_stored) {
          use_baidu_stored = encodeURIComponent(use_baidu_stored);
        }
        use_baidu_sign = use_baidu_response["querySign"];
        if (use_baidu_sign) {
          use_baidu_sign = encodeURIComponent(use_baidu_sign);
        }
      }
    } catch (e) {
      use_baidu_stored = null;
      use_baidu_sign = null;
    }
    if ((!use_baidu_stored) || (!use_baidu_sign)) {
      outer_message("No valid response data from Baidu");
      return;
    }

    var use_baidu_url = null;
    try {
      use_baidu_url = "http://image.baidu.com/pcdutu?";
      use_baidu_url += "queryImageUrl=" + use_baidu_stored;
      use_baidu_url += "&querySign=" + use_baidu_sign;
      use_baidu_url += "&fm=result";
      use_baidu_url += "&uptype=upload_pc";
      use_baidu_url += "&result=result_camera";
      use_baidu_url += "&simid=0,0";
      use_baidu_url += "&querytype=0";
      use_baidu_url += "&fr=pc";
      use_baidu_url += "&pn=0";
    } catch (e) {
      use_baidu_url = null;
    }
    if (!use_baidu_url) {
      outer_message("Can not prepare redirecting to Baidu");
      return;
    }

    redir_to_img_search(use_baidu_url);
    return;
  }

  if (search_prov=="bing") {
    // this case should not occur
    outer_message("Error during dealing towards Bing");
    return;
  }

  if (search_prov=="yandex") {
    var use_yandex_response = img_upload.response;
    var use_yandex_part = null;
    var use_yandex_url = null;
    if (!use_yandex_response) {
      outer_message("No response data from Yandex");
      return;
    }
    try {
      use_yandex_response = JSON.parse(use_yandex_response);
      if (use_yandex_response) {
        use_yandex_part = use_yandex_response["blocks"];
        if (use_yandex_part) {
          use_yandex_part = use_yandex_part[0];
        }
        if (use_yandex_part) {
          use_yandex_part = use_yandex_part["params"];
        }
        if (use_yandex_part) {
          use_yandex_url = use_yandex_part["url"];
        }
      }
    } catch (e) {
      use_yandex_url = null;
    }
    if (!use_yandex_url) {
      outer_message("No valid response data from Yandex");
      return;
    }
    window.location.replace("https://yandex." + yandex_tld +"/images/search?" + use_yandex_url);
    return;
  }

  var ans_redir = img_upload.getResponseHeader("Location");

  if (ans_redir) {
    redir_to_img_search(ans_redir);
    return;
  }

  var ans_load = img_upload.responseURL;
  if (ans_load) {
    redir_to_img_search(ans_load);
    return;
  }

  outer_message("No search-result redirection URL found");
};

// once the canvas is set with the (rescaled) image, sending its data for the search;
// depending on the search provider and the browser, we either send image data via AJAX,
// or we set and submit (html) form with the data (that is redirecting along with that);

var img_data_url_to_blob = function (img_data_url) {
  debug_output("img_data_url_to_blob");

  var url_parts = img_data_url.split(',');
  var mime_info = url_parts[0].match(/:(.*?);/)[1];

  var str_bin = atob(url_parts[1]);
  var str_len = str_bin.length;
  var data_arr = new Uint8Array(str_len);

  for (var ind = str_len - 1; ind >= 0; ind--) {
     data_arr[ind] = str_bin.charCodeAt(ind);
  }

  return new Blob([data_arr], {type:mime_info});
};

var send_image_data = function (can_el, can_width, can_height, search_mode, search_prov) {
  debug_output("send_image_data");

  var aux_data = {};

  var via_baidu = false;
  var via_bing = false;
  var via_google = false;
  var via_tineye = false;
  var via_yandex = false;

  if (search_prov=="baidu") {
    via_baidu = true;
  }
  if (search_prov=="bing") {
    via_bing = true;
  }
  if (search_prov=="google") {
    via_google = true;
  }
  if (search_prov=="tineye") {
    via_tineye = true;
  }
  if (search_prov=="yandex") {
    via_yandex = true;
  }
  if ((!via_google) && (!via_tineye) && (!via_bing) && (!via_baidu) && (!via_yandex)) {
    outer_message("Unknown search provider");
    return;
  }

  var img_blob = null;
  var img_data_url = null;

  var use_png = false;
  var use_qua = 0.95;
  var img_size = can_width * can_height;
  if (img_size <= 10000) {
    use_png = true;
    use_qua = 1.0;
  }
  if (img_size > 40000) {
    use_qua = 0.9;
  }

  var use_mime = (use_png ? "image/png" : "image/jpeg");
  var use_fname = (use_png ? "image.png" : "image.jpg");

  var err_out = false;
  try {
    if (use_png) {
      img_data_url = can_el.toDataURL(use_mime);
    } else {
      img_data_url = can_el.toDataURL(use_mime, use_qua);
    }
  } catch (e) {
    outer_message("Error during taking data");
    img_data_url = null;
    err_out = true;
  }
  if (!img_data_url) {
    if (!err_out) {
      outer_message("Issue during taking data");
    }
    return;
  }

  if (via_tineye || via_baidu || via_yandex) {
    err_out = false;
    try {
      img_blob = img_data_url_to_blob(img_data_url);
    } catch (e) {
      outer_message("Can not take data");
      img_blob = null;
      err_out = true;
    }
    if (!img_blob) {
      if (!err_out) {
        outer_message("Could not take data");
      }
      return;
    }
  }

  if (via_google) {
    img_data_url = img_data_url.substring(img_data_url.indexOf(",")+1);
    img_data_url = img_data_url.replace(/\+/g,"-").replace(/\//g,"_").replace(/\./g,"=");
  }

  if (via_bing) {
    img_data_url = img_data_url.substring(img_data_url.indexOf(",")+1);
  }

  if (img_data_url=="") {
    outer_message("Empty image data");
    return;
  }

  var img_post_url = "";

  if (via_google) {
    img_post_url = "https://www.google.com/searchbyimage/upload";
  }

  if (via_tineye) {
    img_post_url = "https://tineye.com/search";
  }

  if (via_bing) {
    aux_data = {
      "suffix": use_mime.substring(use_mime.indexOf("/")+1),
      "fname": use_fname,
      "length": img_data_url.length
    };
    img_post_url = [
      "https://www.bing.com/images/search?",
      "q=",
      "&view=detailv2",
      "&iss=sbiupload",
      "&FORM=IRSBIQ",
      "&redirecturl=https%3A%2F%2Fwww.bing.com%2Fimages%2Fdiscover%3Fform%3DZ9LH1",
      "&sbifnm=" + aux_data["fname"],
      "&dlen=" + aux_data["length"],
      "&mode=overlay"
    ].join("");
  }

  if (via_baidu) {
    img_post_url = "https://image.baidu.com/pcdutu/a_upload?fr=html5&target=pcSearchImage&needJson=true";
  }

  if (via_yandex) {
    var yx_ww = 0;
    var yx_wh = 0;
    try {
      yx_ww = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth;
      yx_wh = window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight;
    } catch (e) {
      yx_ww = 0;
      yx_wh = 0;
    }

    img_post_url = "https://yandex." + yandex_tld + "/images/search?";
    img_post_url += "rpt=imageview&format=json";
    img_post_url += "&request=%7B%22blocks%22%3A%5B%7B%22block%22%3A%22b-page_type_search-by-image__link%22%7D%5D%7D";
  }

  if (img_upload_runs) {
    if (img_upload) {
      try {img_upload.abort();}
      catch (e) {no_op_function();}
    }
    img_upload_runs = false;
  }

  var img_form = new FormData();

  if (via_google) {

    if (!can_do_post) {
      img_form.append("image_content", img_data_url);
      img_form.append("sbisrc", "cr_1_5_2");
      img_form.append("filename", use_fname);

    } else {
      outer_message("Submitting image data");

      var google_form_el = document.getElementById("form-search-google");
      if (!google_form_el) {
        outer_message("Error at submitting page");
      }
      try {
        google_form_el.elements["image_content"].value = img_data_url;
        google_form_el.elements["filename"].value = use_fname;
        google_form_el.action = img_post_url;
        google_form_el.submit();
      } catch (e) {
        outer_message("Error within submitting page");
      }
      return;

    }
  }

  if (via_tineye) {
    img_form.append("image", img_blob, use_fname);
  }

  if (via_bing) {
    var data_uri_page = "";

    if (!can_do_post) {
      data_uri_page += "<html><head><title>" + "page submitting" + "</title></head>";
      data_uri_page += "<style>";
      data_uri_page += "#message-el { margin-top: 40px; } ";
      data_uri_page += "</style>";
      data_uri_page += "<script language=\"javascript\">";
      data_uri_page += [
        "var initialize_page_submit = function () { document.getElementById(\"form_submit\").submit(); }; ",
        "if ((document.readyState == \"loading\") || (document.readyState == \"uninitialized\")) { ",
        "  if (document.addEventListener) { ",
        "    document.addEventListener(\"DOMContentLoaded\", initialize_page_submit); ",
        "  } else { ",
        "    window.onload = initialize_page_submit; ",
        "  } ",
        "} else { ",
        "  initialize_page_submit(); ",
        "}"].join("");
      data_uri_page += "</" + "script>";
      data_uri_page += "<body>";
      data_uri_page += "<center id=\"message-el\"><b>" + "Submitting the image" + "</b></center>";
      data_uri_page += "<html><head><title>"+"testing"+"</title></head><body>";
      data_uri_page += "<form id=\"form_submit\" method=\"POST\" action=\"" + img_post_url + "\" enctype=\"multipart/form-data\">";
      data_uri_page += "<input type=\"hidden\" name=\"imgurl\" value=\"\">";
      data_uri_page += "<input type=\"hidden\" name=\"cbir\" value=\"sbi\">";
      data_uri_page += "<input type=\"hidden\" name=\"imageBin\" value=\"" + img_data_url + "\">";
      data_uri_page += "</form></body></html>";

      data_uri_page = "data:text/html;charset=utf-8;base64," + window.btoa(data_uri_page);

      submit_to_bing_via_data_uri_set = data_uri_page;
      should_ask_user_on_data_uri();
      return;

    } else {
      outer_message("Submitting image data");

      var bing_form_el = document.getElementById("form-search-bing");
      if (!bing_form_el) {
        outer_message("Error at submitting page");
      }
      try {
        bing_form_el.elements["imageBin"].value = img_data_url;
        bing_form_el.action = img_post_url;
        bing_form_el.submit();
      } catch (e) {
        outer_message("Error within submitting page");
      }
      return;

    }
  }

  if (via_baidu) {
    img_form.append("file", img_blob, use_fname);

    img_form.append("pos", "upload");
    img_form.append("uptype", "upload_pc");
    img_form.append("fm", "index");
  }

  if (via_yandex) {
    img_form.append("upfile", img_blob, use_fname);
  }

  img_upload = new XMLHttpRequest();
  if (!img_upload) {
    outer_message("Error during submitting preparation");
    return;
  }
  img_upload.open('POST', img_post_url, true);

  img_upload.onreadystatechange = function () {
    if (!img_upload) {
      return;
    }
    var req_done = img_upload.DONE || 4;
    if (img_upload.readyState === req_done) {
      if (img_upload.status!=200) {
        outer_message("Image could not be submitted");
        utilize_image_upload(false, search_prov, aux_data);
        return;
      }
      utilize_image_upload(true, search_prov, aux_data);
    }
  };

  outer_message("Submitting image data");
  img_upload.send(img_form);

};

// once image is received, putting it into canvas,
// possibly with rescaling (if the image is too big);
// and if requested, the image is grayscaled too;

var img_request = null;
var img_request_runs = false;
var waiting_yandex = false;

var img_use_width = 0;
var img_use_height = 0;

var can_img = null;
var img_url = null;

// this could be done via canvas filters too,
// though not sure about compatibility of that;
// thus using this hand-made way by now;
var make_grayscale = function (can_el, can_width, can_height) {
  debug_output("make_grayscale");

  var can_ctx = can_el.getContext("2d");

  var img_data = can_ctx.getImageData(0, 0, can_width, can_height);
  var arr_data = img_data.data;
  var arr_len = arr_data.length;

  var avg_val;
  for (var ind = 0; ind < arr_len; ind += 4) {
    avg_val = (arr_data[ind] + arr_data[ind + 1] + arr_data[ind + 2]) / 3;
    arr_data[ind] = avg_val;
    arr_data[ind+1] = avg_val;
    arr_data[ind+2] = avg_val;
  }
  can_ctx.putImageData(img_data, 0, 0);

};

var utilize_image_from_url = function (ans_state, search_mode, search_prov) {
  debug_output("utilize_image_from_url");
  outer_message("Preparing image data");

  img_request_runs = false;
  if (!img_request) {
    return;
  }
  if (!ans_state) {
    img_request = null;
    return;
  }

  var img_data = null;
  if ("mozResponseArrayBuffer" in img_request) {
    img_data = img_request.mozResponseArrayBuffer;
  }
  if ((!img_data) && img_request.hasOwnProperty("responseType") && img_request.responseType == "arraybuffer") {
    img_data = img_request.response;
  }
  if ((!img_data) && ("response" in img_request)) {
    img_data = img_request.response;
  }
  img_request = null;
  if (!img_data) {
    outer_message("Has not received image data");
    return;
  }

  var err_out = false;
  var img_blob = null;
  try {
    img_blob = new Blob([img_data], {type: "application/octet-binary"});
  } catch (e) {
    outer_message("Can not utilize image data");
    img_blob = null;
    err_out = true;
  }
  if (!img_blob) {
    if (!err_out) {
      outer_message("Issue during image data utilization");
    }
    return;
  }

  img_url = null;
  try {
    img_url = URL.createObjectURL(img_blob);
  } catch (e) {
    outer_message("Can not use image data");
    img_url = null;
    err_out = true;
  }
  if (!img_url) {
    if (!err_out) {
      outer_message("Issue during image data use");
    }
    return;
  }

  var can_el = document.getElementById("canvas-el");
  if (!can_el) {
    outer_message("Error during image-data preparation");
    return;
  }
  var can_aux_el = null;

  err_out = false;
  var can_ctx = null;
  try {
    can_ctx = can_el.getContext("2d");
  } catch (e) {
    outer_message("Error during image-data preparing");
    can_ctx = null;
    err_out = true;
  }
  if (!can_ctx) {
    if (!err_out) {
      outer_message("Issue during image data preparing");
    }
    return;
  }

  can_img = new Image();
  can_img.onerror = function () {
    outer_message("Could not prepare image data");
    return;
  }
  can_img.onload = function () {
    var orig_width = can_img.width;
    var orig_height = can_img.height;
    if ((orig_width <= 0) || (orig_height <= 0)) {
      outer_message("Invalid (empty) image data");
      return;
    }
    var orig_size = orig_width * orig_height;

    img_use_width = orig_width;
    img_use_height = orig_height;

    var got_err = false;
    try {
      if (orig_size <= max_canvas_size) {
        // small/moderate images used directly
        can_el.width = img_use_width;
        can_el.height = img_use_height;
        if (search_mode=="file:flip") {
          can_ctx.setTransform(-1, 0, 0, 1, img_use_width, 0);
        }
        can_ctx.drawImage(can_img, 0, 0);
      } else {
        // large images should be rescaled down
        var scale_f = Math.sqrt(max_canvas_size / orig_size);
        img_use_width = orig_width * scale_f;
        img_use_height = orig_height * scale_f;
        can_el.width = img_use_width;
        can_el.height = img_use_height;
        img_use_width = can_el.width;
        img_use_height = can_el.height;
        var use_img = can_img;
        var one_step = 0.5;
        var next_size = orig_size * one_step;
        var curr_w = orig_width;
        var curr_h = orig_height;

        if (next_size >= max_canvas_size) {
          // rescaling at steps for way too much large images
          can_aux_el = document.getElementById("canvas-aux-el");
          if (!can_aux_el) {
            outer_message("Error during image-data preparation setting");
            return;
          }
          var can_aux_ctx = null;
          err_out = false;
          try {
            can_aux_ctx = can_aux_el.getContext("2d");
          } catch (e) {
            outer_message("Error during preparing image scaling");
            can_aux_ctx = null;
            err_out = true;
          }
          if (!can_aux_ctx) {
            if (!err_out) {
              outer_message("Issue during preparing image scaling");
            }
            return;
          }
          can_aux_el.width = orig_width;
          can_aux_el.height = orig_height;

          var one_step_side = Math.sqrt(one_step);
          var next_w = curr_w * one_step_side;
          var next_h = curr_h * one_step_side;
          while (next_size > max_canvas_size) {
            can_aux_ctx.drawImage(use_img, 0, 0, curr_w, curr_h, 0, 0, next_w, next_h);
            use_img = can_aux_el;
            next_size = next_size * one_step;
            curr_w = next_w;
            next_w = next_w * one_step_side;
            curr_h = next_h;
            next_h = next_h * one_step_side;
          }
        }

        if (search_mode=="file:flip") {
          can_ctx.setTransform(-1, 0, 0, 1, img_use_width, 0);
        }
        can_ctx.drawImage(use_img, 0, 0, curr_w, curr_h, 0, 0, img_use_width, img_use_height);
        can_aux_el = null;
      }
    } catch (e) {
      outer_message("Can not prepare image data");
      got_err = true;
    }
    if (got_err) {
      return;
    }

    if (under_debug) {
      var can_hld = document.getElementById("canvas-holder");
      if (can_hld) {
        can_hld.style.display = "block";
      }
    }

    img_url = null;
    try {
      window.URL.revokeObjectURL(can_img.src);
    } catch (e) {
      no_op_function();
    }
    can_img = null;

    if (search_mode=="file:gray") {
      make_grayscale(can_el, img_use_width, img_use_height);
    }

    send_image_data(can_el, img_use_width, img_use_height, search_mode, search_prov);
  };
  can_img.src = img_url;
};

// taking the image from the provided URL

var load_image_from_url = function (search_mode, search_prov, img_url) {
  debug_output("load_image_from_url");

  if (img_url.substring(0, 4)!="http") {
    outer_message("Non-recognized image URL");
    return;
  }

  if (img_request_runs) {
    if (img_request) {
      try {img_request.abort();}
      catch (e) {no_op_function();}
    }
    img_request_runs = false;
  }

  img_request = new XMLHttpRequest();
  if (!img_request) {
    outer_message("Error during image-download preparation");
    return;
  }
  img_request.open('GET', img_url, true);
  img_request.responseType = "arraybuffer";
  img_request.setRequestHeader("Accept", "*/*");

  img_request.onreadystatechange = function () {
    if (!img_request) {
      return;
    }
    var req_done = img_request.DONE || 4;
    if (img_request.readyState === req_done) {
      if (img_request.status!=200) {
        outer_message("Could not download the image: " + img_request.status);
        utilize_image_from_url(false, search_mode, search_prov);
        return;
      }
      utilize_image_from_url(true, search_mode, search_prov);
    }
  };

  img_request.send(null);
  img_request_runs = true;
  outer_message("Downloading the image");
};

// loading additional info if needed; by now for Yandex only;

var load_auxiliar_info = function (search_mode, search_prov, img_url) {
  debug_output("load_auxiliar_info");

  // had previously used it on Yandex, though (apparently) do/should not use it now;
  // still keeping the function here, if we should use it at some cases again;
};

// setting initial properties on stuff where we may ask user later;
// by now, this is for the Bing case under Firefox only;

var ext_on_self_call = function (info) {
  debug_output("ext_on_self_call");

  if (ext_holder.runtime.lastError) {
    debug_output(ext_holder.runtime.lastError);
  }

  ext_on_self_got(info);
}

var ext_on_self_got = function (info) {
  debug_output("ext_on_self_got");

  var hp_key = "homepageUrl";
  if (info && (hp_key in info) && info[hp_key]) {
    var hp_simple_el = document.getElementById("file-notice-simple");
    var hp_link_el = document.getElementById("file-notice-link");
    var hp_inner_el = document.getElementById("file-notice-link-inner");
    if (hp_simple_el && hp_link_el && hp_inner_el) {
      hp_simple_el.style.display = "none";
      hp_link_el.style.display = "inline-block";
      hp_inner_el.href = info[hp_key];
    }
  }
};

var ext_on_self_error = function (info_error) {
  debug_output("ext_on_self_error");

  debug_output(info_error);

  no_op_function();
};

var initialize_user_actions = function () {
  debug_output("initialize_user_actions");

  var aux_el;
  var ind;

  var els_to_hide = [
    "bing-parts-holder"
  ];
  for (ind = els_to_hide.length - 1; ind >= 0; ind--) {
    aux_el = document.getElementById(els_to_hide[ind]);
    if (aux_el) {
      aux_el.style.display = "none";
    }
  }

  var bing_submit_el = document.getElementById("bing-submit-el");
  if (bing_submit_el) {
    bing_submit_el.onclick = submit_to_bing_via_data_uri_action;
  }

  var els_to_bind = [
    "bing-not-ask-el",
    "bing-not-ask-cb"
  ];
  for (ind = els_to_bind.length - 1; ind >= 0; ind--) {
    aux_el = document.getElementById(els_to_bind[ind]);
    if (aux_el) {
      aux_el.onclick = next_on_bing_data_uri_to_not_ask_action;
    }
  }

  if (ext_uses_browser) {
    ext_holder.management.getSelf().then(ext_on_self_got, ext_on_self_error);
  } else {
    ext_holder.management.getSelf(ext_on_self_call);
  }
};

// making onset of the stuff

var max_canvas_size = 360000;

var initialize_page_state = function () {
  debug_output("initialize_page_state");

  document.title = ext_holder.i18n.getMessage("extensionName");

  var page_url = window.location.href.toString();
  var hash_pos = page_url.indexOf("#");
  if (hash_pos == -1) {
    return;
  }
  var suffix_part = page_url.substring(hash_pos+1);

  var hash_pos = suffix_part.indexOf("#");
  if (hash_pos == -1) {
    return;
  }
  var search_mode = suffix_part.substring(0, hash_pos).toLowerCase();

  suffix_part = suffix_part.substring(hash_pos+1);
  var hash_pos = suffix_part.indexOf("#");
  if (hash_pos == -1) {
    return;
  }
  var search_prov = suffix_part.substring(0, hash_pos).toLowerCase();
  var img_url = suffix_part.substring(hash_pos+1);

  if ((search_prov=="bing") && (!can_do_post)) {
    max_canvas_size = 90000;
  }

  load_auxiliar_info(search_mode, search_prov, img_url);
  load_image_from_url(search_mode, search_prov, img_url);
  initialize_user_actions();
};

if ((document.readyState == "loading") || (document.readyState == "uninitialized")) {
  if (document.addEventListener) {
    document.addEventListener("DOMContentLoaded", initialize_page_state);
  } else {
    window.onload = initialize_page_state;
  }
} else {
  initialize_page_state();
}

